﻿using Infragistics.Controls.Menus;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Infragistics.DragDrop;

namespace XamDataTreeVertScroll
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        //the number of levels in the hierarchy
        private int treeDepth = 2;
        //the number of children of each node
        private int levelNodeCount = 5;
        private SimpleNodeDataSource viewModel;

        public MainWindow()
        {
            InitializeComponent();
            this.Loaded += OnSampleLoaded;
        }

        void OnSampleLoaded(object sender, RoutedEventArgs e)
        {
            viewModel = new SimpleNodeDataSource();
            DataContext = viewModel;
            LoadDataSource();
        }


        private void LoadDataSource()
        {
            IList<SimpleNode> nodes = new List<SimpleNode>();
            SetNodes(nodes, 0);
            viewModel.SimpleNodes = nodes;
        }

      
        private void SetNodes(IList<SimpleNode> nodes, int level)
        {
            if (level == treeDepth)
                return;
            SimpleNode sn;
            for (int i = 0; i < levelNodeCount; ++i)
            {
                sn = new SimpleNode();
                sn.Text = "Node" + (i + 1).ToString() + " : " + "Level" + level.ToString();
                sn.Children = new List<SimpleNode>();
                SetNodes(sn.Children, level + 1);
                nodes.Add(sn);
            }
        }
    }

    public class ObservableModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void OnPropertyChanged(string propertyName)
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        
    }


    public class SimpleNodeDataSource : ObservableModel
    {
        private IEnumerable<SimpleNode> simpleNodes;
        public IEnumerable<SimpleNode> SimpleNodes
        {
            get { return simpleNodes; }
            set
            {
                if (simpleNodes != value)
                {
                    simpleNodes = value;
                    this.OnPropertyChanged("SimpleNodes");
                }
            }
        }
    }

    public class SimpleNode : ObservableModel
    {
        public static int NodesCount = 0;
        private string text;
        private List<SimpleNode> children;
        private bool isExpanded = false;

        public SimpleNode()
        {
            NodesCount++;
        }

        public bool IsExpanded
        {
            get
            {
                return isExpanded;
            }
            set
            {
                isExpanded = value;
                this.OnPropertyChanged("IsExpanded");
            }
        }
        public List<SimpleNode> Children
        {
            get { return children; }
            set
            {
                if (children != value)
                {
                    children = value;
                    this.OnPropertyChanged("Children");
                }
            }
        }
        public string Text
        {
            get { return text; }
            set
            {
                if (text != value)
                {
                    text = value;
                    this.OnPropertyChanged("Text");
                }
            }
        }
    }

    public class XamDataTreeEx : XamDataTree
    {
        public XamDataTreeEx()
        {
            
        }

        protected override void OnNodeDraggingStart(DragDropStartEventArgs e)
        {
            base.OnNodeDraggingStart(e);
            var node = e.Data as XamDataTreeNode;

            if (node != null && node.NodeLayout is NodeLayoutEx)
            {
                e.DragTemplate = ((NodeLayoutEx)node.NodeLayout).DragTemplate;
            }
        }
    }

}
